(*                                                                            *)
(* 	Display free system resources in three meter controls                 *)
(*                                                                            *)
(*	SYSMET.PAS                                                            *)
(*                                                                            *)
(*	Copyright (c) Luk Vermeulen 1992                                      *)
(*                                                                            *)

PROGRAM SysMet;

{$D System Resource Meter}
{$R SYSMET}

USES    WinTypes, WinProcs, WObjects, Strings, Win31, CommDlg,
	CPalette, CPObject;

CONST
        { Miscellaneous constants }
	AppName         = 'System Resource Meter';
	TimerID		= 1;

        { Application menu commands }
	CM_About	= 101;
        CM_Purpose	= 102;
	CM_Credits	= 103;
	CM_Exit		= 104;

	{ Meter menu commands }
	CM_MeterHelp	= 105;
	CM_MeterFont	= 106;
	CM_MeterColor	= 107;

	{ Dialog control IDs }
	IDC_Static1	= 101;
	IDC_Static2	= 102;
	IDC_Static3	= 103;
	IDC_Static4	= 104;
        IDC_Hidden	= 105;

        { Meter control IDs }
	IDC_UserMeter	= 101;
	IDC_GDIMeter	= 102;
	IDC_MemMeter	= 103;

	{ Static control IDs }
	IDC_UserStatic	= 104;
	IDC_GDIStatic	= 105;
	IDC_MemStatic	= 106;

VAR
	TimerSet,
	MeterWidth, MeterHeight, Between,
        ScreenWidth, ScreenHeight,
	IconWidth, IconHeight: Longint;
	i, Pix, Offset, wBegin, BarWidth, BarHeight, MaxBarHeight: Word;
	FreeResources, TempFreeResources: Array[0..2] of Word;

TYPE
	PMeter = ^TMeter;
	TMeter = OBJECT(TCPMeter)
		Menu: hMenu;
		HelpText: PChar;
		MeterFont: hFont;
		MeterLogFont: TLogFont;

		CONSTRUCTOR Init(AParent: PWindowsObject;
				AnID,
				X, Y, W, H: Integer;
				Limits: Word;
				HelpString: PChar);
		DESTRUCTOR Done; VIRTUAL;
		PROCEDURE MakeInitialFont(VAR AFont: hFont); VIRTUAL;
		PROCEDURE SetupWindow; VIRTUAL;
		PROCEDURE WMRButtonDown(VAR Msg: TMessage);
			VIRTUAL WM_First + WM_RButtonDown;
		PROCEDURE CMHelp(VAR Msg: TMessage);
			VIRTUAL CM_First + CM_MeterHelp;
		PROCEDURE CMFont(VAR Msg: TMessage);
			VIRTUAL CM_First + CM_MeterFont;
		PROCEDURE CMColor(VAR Msg: TMessage);
			VIRTUAL CM_First + CM_MeterColor;
		end;

	PCenterDialog = ^TCenterDialog;
	TCenterDialog = OBJECT(TDialog)
			PROCEDURE SetupWindow; VIRTUAL;
			end;

	PAboutDialog = ^TAboutDialog;
	TAboutDialog = OBJECT(TCenterDialog)
			PROCEDURE HiddenButton(VAR Msg: TMessage);
				VIRTUAL ID_First + IDC_Hidden;
			end;

	PSysMetWin = ^TSysMetWin;
	TSysMetWin = OBJECT(TWindow)
			AltWindowTitle: PChar;
			Menu, SysMenu: HMenu;
			UserMeter, GDIMeter, MemMeter: PMeter;
			UserStatic, GDIStatic, MemStatic: PCPStatic;
			ScreenWidth, ScreenHeight: Longint;

			CONSTRUCTOR Init(AParent: PWindowsObject;
					ATitle: PChar);
			DESTRUCTOR Done; VIRTUAL;
			PROCEDURE WMCreate(VAR Msg: TMessage);
				VIRTUAL WM_First + WM_Create;
			PROCEDURE WMDestroy(VAR Msg: TMessage);
				VIRTUAL WM_First + WM_Destroy;
			FUNCTION GetClassName : PChar; VIRTUAL;
			PROCEDURE GetWindowClass(VAR AWndClass: TWndClass);
				VIRTUAL;
			PROCEDURE SetupWindow; VIRTUAL;
			PROCEDURE WMQueryOpen(VAR Msg: TMessage);
				 VIRTUAL WM_First + WM_QueryOpen;
			PROCEDURE WMControlAttribs(VAR Msg: TMessage);
				 VIRTUAL WM_First + WM_CtlAttribs;
			PROCEDURE WMControlColor(VAR Msg: TMessage);
				VIRTUAL WM_First + WM_CtlColor;
			FUNCTION ResourceStatusChanged : Bool;
			PROCEDURE MakeAltWindowTitle;
			PROCEDURE WMTimer(VAR Msg: TMessage);
				VIRTUAL WM_First + WM_Timer;
			PROCEDURE WMSize(VAR Msg: TMessage);
				VIRTUAL WM_First + WM_Size;
			PROCEDURE Paint(DC: HDC; VAR PS: TPaintStruct); VIRTUAL;
			PROCEDURE DrawMyIcon(TheDC: hDC);
			PROCEDURE WMSysCommand(VAR Msg: TMessage);
				VIRTUAL WM_First + WM_SysCommand;
			PROCEDURE WMNCRButtonDown(VAR Msg: TMessage);
				VIRTUAL WM_First + WM_NCRButtonDown;
			PROCEDURE WMRButtonDown(VAR Msg: TMessage);
				VIRTUAL WM_First + WM_RButtonDown;
			PROCEDURE CMAbout(VAR Msg: TMessage);
				VIRTUAL CM_First + CM_About;
			PROCEDURE CMPurpose(VAR Msg: TMessage);
				VIRTUAL CM_First + CM_Purpose;
			PROCEDURE CMCredits(VAR Msg: TMessage);
				VIRTUAL CM_First + CM_Credits;
			PROCEDURE CMQuit(VAR Msg: TMessage);
				VIRTUAL CM_First + CM_Exit;
			end;

	TSysMetApp = OBJECT(TApplication)
			PROCEDURE InitMainWindow; VIRTUAL;
			end;

(* -------------------------------------------------------------------------- *)

{ TMeter }

CONSTRUCTOR TMeter.Init(AParent:PWindowsObject;
			AnID,
			X,Y,W,H:Integer;
			Limits:Word;
			HelpString: PChar);
BEGIN
        { Initialize meter control and set style to our liking }
	TCPMeter.Init(AParent,AnID,X,Y,W,H,Limits);
	SetStyle(WS_Child OR WS_Visible OR WS_Group OR MTS_Lowered OR MTS_Vert);

        { Prepare Object Inspector's local menu }
	HelpText := HelpString;
	Menu := LoadMenu(HInstance,'Meter');
	Menu := GetSubMenu(Menu,0);

        { Initialize properties }
	RemColorRef := RGB(192,192,192);
	MeterFont := 0;
END;

DESTRUCTOR TMeter.Done; 
BEGIN
        { Clean up associated resources }
	if MeterFont <> 0 then DeleteObject(MeterFont);
	DestroyMenu(Menu);
	TCPMeter.Done; 
END;

PROCEDURE TMeter.MakeInitialFont(VAR AFont: hFont);
BEGIN
        { Make initial font for meter control }
	if AFont <> 0 then DeleteObject(AFont);
	FillChar(MeterLogFont,SizeOf(TLogFont),#0);
	With MeterLogFont do
		begin
		lfHeight := 20;
		lfWeight := 700;
		lfItalic := 1;
		lfUnderline := 0;
		lfOutPrecision := Out_Stroke_Precis;
		lfClipPrecision := Clip_Stroke_Precis;
		lfQuality := Default_Quality;
		lfPitchAndFamily := Variable_Pitch;
		StrCopy(lfFaceName,'Arial');
		end;
	AFont := CreateFontIndirect(MeterLogFont);
END;

PROCEDURE TMeter.SetupWindow;
BEGIN
        { Prepare meter control for screen }
	TCPMeter.SetupWindow;
	MakeInitialFont(TextFont);
END;

PROCEDURE TMeter.WMRButtonDown(VAR Msg: TMessage);
VAR	Point: TPoint;
BEGIN
	{ Pop up Object Inspector's local menu }
	Point := MakePoint(Msg.lParam);
	ClientToScreen(HWindow,Point);
        TrackPopupMenu(Menu,0,Point.X,Point.Y,0,HWindow,NIL);
END;

PROCEDURE TMeter.CMHelp(VAR Msg: TMessage);
BEGIN
	{ What's this meter for? }
	MessageBox(HWindow,HelpText,'Help',MB_IconQuestion OR MB_OK);
END;

PROCEDURE TMeter.CMFont(VAR Msg: TMessage);
VAR
	Font: TChooseFont;
	Colors: LongInt;
	Style: Array [0..100] of Char;
	TempFont: TLogFont;
BEGIN
	{ Prepare common dialog box }
	FillChar(Font,SizeOf(Font),#0);
	With Font do
		begin
		lStructSize := SizeOf(TChooseFont);
		hwndOwner := HWindow;
		lpLogFont := @MeterLogFont;
		Flags := CF_ScreenFonts OR CF_Effects OR CF_InitToLogFontStruct;
		RGBColors := RemColorRef;
		lpszStyle := Style;
		end;

	{ Pop up common dialog box }
	if NOT ChooseFont(Font) then Exit;

	{ First clean up obsolete properties }
	if MeterFont <> 0 then DeleteObject(MeterFont);
	if TextFont <> 0 then DeleteObject(TextFont);

	{ Set properties to user's selections }
	RemColorRef:= Font.RGBColors;
	MeterFont := CreateFontIndirect(MeterLogFont);
        TextFont := MeterFont;

	{ Repaint meter control }
	InvalidateRect(HWindow,NIL,TRUE);
END;

PROCEDURE TMeter.CMColor(VAR Msg: TMessage);
CONST
	ColorList: Array[0..15] of Longint =
		(
		$000000, $100000, $200000, $300000,
		$400000, $500000, $600000, $700000,
		$800000, $900000, $A00000, $B00000,
		$C00000, $D00000, $E00000, $F00000
		);
VAR
	Color: TChooseColor;
	i: Integer;
BEGIN
        { Prepare common dialog box }
	with Color do
		begin
		HWndOwner := HWindow;
		lStructSize := Sizeof(TChooseColor);
		RGBResult := BarColorRef;
		lpCustColors := @ColorList;
		Flags := CC_FullOpen or CC_RGBInit;
		end;

	{ Pop up common dialog box }
	if NOT ChooseColor(Color) then Exit;

	{ Set properties to user's selections }
	BarColorRef := Color.RGBResult;

        { Repaint meter control }
	InvalidateRect(HWindow,NIL,TRUE);
END;

(* -------------------------------------------------------------------------- *)

{ TCenterDialog }

PROCEDURE TCenterDialog.SetupWindow;
VAR
	Rect: TRect;
	X, Y: Integer;
BEGIN
	{ Reposition dialog box in center of screen }
	TDialog.SetupWindow;					
	X := GetSystemMetrics(SM_CXScreen) DIV 2;		
	Y := GetSystemMetrics(SM_CYScreen) DIV 2;		
	GetWindowRect(HWindow, Rect);				
	With Rect do SetWindowPos(HWindow,0,
			x -  ((Right-Left) DIV 2), y - ((Bottom-Top) DIV 2),
			Rect.Right, Rect.Bottom,
			SWP_NoSize OR SWP_NoZOrder);
END;

(* -------------------------------------------------------------------------- *)

{ TAboutDialog }

PROCEDURE TAboutDialog.HiddenButton(VAR Msg: TMessage);
BEGIN
	{ Respond to Alt-i keystroke in about dialog box as in TPW's }
	SetDlgItemText(HWindow,IDC_Static1,'Another nice product');
	SetDlgItemText(HWindow,IDC_Static2,'from');
	SetDlgItemText(HWindow,IDC_Static3,'Violent Fart, Inc.');
	SetDlgItemText(HWindow,IDC_Static4,'Copyright  Lord Cornflake 1992');
END;

(* -------------------------------------------------------------------------- *)

{ TSysMetWin }

CONSTRUCTOR TSysMetWin.Init(AParent: PWindowsObject; ATitle: PChar);
BEGIN
        { Initialize object }
	TWindow.Init(AParent,ATitle);
	GetMem(AltWindowTitle, 15);

	{ Prepare Object Inspector's local menu }
	Menu := LoadMenu(HInstance,'Application');
	Menu := GetSubMenu(Menu,0);

        { Initialize variables for icon drawing }
	IconWidth := GetSystemMetrics(SM_CXIcon);
	IconHeight := GetSystemMetrics(SM_CYIcon);
	Pix := (IconWidth DIV 16) + 1;
	Offset := (IconHeight DIV 8) + 1;
	BarWidth := IconWidth DIV 4;
	MaxBarHeight := (IconHeight * 3) DIV 4;

	{ Meter & Static controls for free USER resources }
	UserMeter := New(PMeter,Init(@Self,IDC_UserMeter,
					0, 0, MeterWidth, MeterHeight, 100,
		'This bar indicates the amount of free USER resources.'));

	UserStatic := New(PCPStatic,Init(@Self,IDC_UserStatic,'User',0,0,0,0,10));
	UserStatic^.Attr.Style := UserStatic^.Attr.Style AND (NOT SS_Left)
							OR SS_Center;

	{ Meter & Static controls for free GDI resources }
	GDIMeter := New(PMeter,Init(@Self,IDC_GDIMeter,
					0, 0, MeterWidth, MeterHeight, 100,
		'This bar indicates the amount of free GDI resources.'));

	GDIStatic := New(PCPStatic,Init(@Self,IDC_GDIStatic,'GDI',0,0,0,0,10));
	With GDIStatic^ do Attr.Style := Attr.Style AND (NOT SS_Left)
							OR SS_Center;

	{ Meter & Static controls for free SYSTEM resources }
	MemMeter := New(PMeter,Init(@Self,IDC_MemMeter,
					0, 0, MeterWidth, MeterHeight, 100,
		'This bar indicates the amount of free SYSTEM resources.'));

	MemStatic := New(PCPStatic,Init(@Self,IDC_MemStatic,'Memory',0,0,0,0,10));
	MemStatic^.Attr.Style := MemStatic^.Attr.Style AND (NOT SS_Left)
							OR SS_Center;

	{ Make sure user can see active applications' icons }
	Attr.X := 0;
	Attr.Y := 0;
	Attr.W := GetSystemMetrics(SM_CXFullScreen);
	Attr.H := GetSystemMetrics(SM_CYFullScreen) -
			(2 * GetSystemMetrics(SM_CYIcon) );
END;

DESTRUCTOR TSysMetWin.Done;
BEGIN
        { Free up resources }
	FreeMem(AltWindowTitle, 15);
	DestroyMenu(Menu);
	DestroyMenu(SysMenu);
	TWindow.Done;
END;

PROCEDURE TSysMetWin.WMCreate(VAR Msg: TMessage);
BEGIN
        { Initialize timer for updating of meter controls }
	TWindow.WMCreate(Msg);
	TimerSet := SetTimer(HWindow,TimerID,1000,NIL);
	if TimerSet = 0 then
		begin
		MessageBox(HWindow,'Error initializing application','Error',
				MB_IconExclamation OR MB_OK);
                CloseWindow;
		end;
END;

PROCEDURE TSysMetWin.WMDestroy(VAR Msg: TMessage);
BEGIN
	{ Destroy timer }
	if TimerSet <> 0 then KillTimer(HWindow,TimerID);
	TWindow.WMDestroy(Msg);
END;

FUNCTION TSysMetWin.GetClassName : PChar;
BEGIN
	{ This class name is as unique as they come }
	GetClassName := 'LordCornflake_SysMetWin';
END;

PROCEDURE TSysMetWin.GetWindowClass(var AWndClass: TWndClass);
BEGIN
	{ Set background color and tell Windows we'll paint the icon ourselves }
	TWindow.GetWindowClass(AWndClass);
	AWndClass.hBrBackground := Color_BtnShadow;
	AWndClass.hIcon := 0;
END;

PROCEDURE TSysMetWin.SetupWindow;
BEGIN
	TWindow.SetupWindow;

	{ Modify system menu }
	SysMenu := GetSystemMenu(HWindow,FALSE);
	AppendMenu(SysMenu, MF_Separator, 0, nil);
	AppendMenu(SysMenu, MF_String, CM_About, '&About...');


	{ Set initial meter colors to Belgian flag }
	UserMeter^.BarColorRef := RGB(  0,   0, 0);	{ Solid black  }
	GDIMeter^.BarColorRef  := RGB(255, 255, 0);     { Solid yellow }
	MemMeter^.BarColorRef  := RGB(255,   0, 0);     { Solid red    }
END;

PROCEDURE TSysMetWin.WMQueryOpen(VAR Msg: TMessage);
BEGIN
	{ Set caption back to app's name }
	SetWindowText(HWindow,'System Resource Meter');
	DefWndProc(Msg);
END;

PROCEDURE TSysMetWin.WMControlAttribs(VAR Msg: TMessage);
BEGIN
	{ Make sure we can modify attributes for our meter controls }
	SendMessage(HWND(Msg.wParam),WM_CtlAttribs,Msg.wParam,Msg.lParam);
END;

PROCEDURE TSysMetWin.WMControlColor(VAR Msg: TMessage);
BEGIN
	{ Make sure we can modify colors for our static controls }
	if Msg.lParamHi = ctlColor_Static then
    		begin
		SetTextColor(Msg.WParam, 0);			
		SetBkMode(Msg.WParam, Transparent);
		Msg.Result := GetStockObject(null_Brush);
		end;
END;

FUNCTION TSysMetWin.ResourceStatusChanged : Bool;
BEGIN
	{ Get free resources status }
	ResourceStatusChanged := FALSE;
	TempFreeResources[0] := GetFreeSystemResources(GFSR_UserResources);
	TempFreeResources[1] := GetFreeSystemResources(GFSR_GDIResources);
	TempFreeResources[2] := GetFreeSystemResources(GFSR_SystemResources);

	{ And check if they've changed since last time }
        for i := 0 to 2 do
        	begin
		if FreeResources[i] <> TempFreeResources[i] then
			begin
			FreeResources[i] := TempFreeResources[i];
			ResourceStatusChanged := TRUE;
			end;
		end;
END;

PROCEDURE TSysMetWin.MakeAltWindowTitle;
VAR	ArgList: Array[0..2] of Integer;
BEGIN
        { Set alternate caption to meter values }
	ArgList[0] := UserMeter^.GetProgress;
	ArgList[1] := GDIMeter^.GetProgress;
	ArgList[2] := MemMeter^.GetProgress;
	WVSPrintf(AltWindowTitle,'U%d-G%d-M%d',ArgList);
END;

PROCEDURE TSysMetWin.WMTimer(VAR Msg: TMessage);
BEGIN
	{ Prevent flashing of meter controls by checking for change first }
	if ResourceStatusChanged then
        	begin
        	{ Update meter controls }
		UserMeter^.SetProgress(FreeResources[0]);
		GDIMeter^.SetProgress(FreeResources[1]);
		MemMeter^.SetProgress(FreeResources[2]);

		{ Update alternate window title & iconic meter controls }
        	if IsIconic(HWindow) then
			begin
                        MakeAltWindowTitle;
			SetWindowText(HWindow, AltWindowTitle);
			InvalidateRect(HWindow, nil, TRUE);
			end;
		end;
END;

PROCEDURE TSysMetWin.WMSize(VAR Msg: TMessage);
BEGIN
        TWindow.WMSize(Msg);
	if Msg.wParam <> Size_Minimized then
		begin
		ScreenWidth := Msg.lParamLo;
		ScreenHeight := Msg.lParamHi;
		end;

        { Set meter position variables relative to window }
        MeterWidth := ScreenWidth DIV 4;
        MeterHeight := (ScreenHeight DIV 8) * 6;
	Between := MeterWidth DIV 4;

	{ Adjust position of Meter & Static controls for free USER resources }
	SetWindowPos(UserMeter^.HWindow, 0,
			Between, ScreenHeight DIV 8,
			MeterWidth, MeterHeight, SWP_NoZOrder); 
	SetWindowPos(UserStatic^.HWindow,0,
			Between,
			(ScreenHeight DIV 24) * 22,
			MeterWidth,
			(ScreenHeight DIV 24) * 23,
			SWP_NoZOrder);

	{ Adjust position of Meter & Static controls for free GDI resources }
	SetWindowPos(GDIMeter^.HWindow, 0,
			(Between * 2) + MeterWidth, ScreenHeight DIV 8,
			MeterWidth, MeterHeight, SWP_NoZOrder);
	SetWindowPos(GDIStatic^.HWindow,0,
			(Between * 2) + MeterWidth,
			(ScreenHeight DIV 24) * 22,
			MeterWidth,
			(ScreenHeight DIV 24) * 23,
			SWP_NoZOrder);

	{ Adjust position of Meter & Static controls for free SYSTEM resources }
	SetWindowPos(MemMeter^.HWindow, 0,
			(Between * 3) + (MeterWidth * 2), ScreenHeight DIV 8,
			MeterWidth, MeterHeight, SWP_NoZOrder);
	SetWindowPos(MemStatic^.HWindow,0,
			(Between * 3) + (MeterWidth * 2),
			(ScreenHeight DIV 24) * 22,
			MeterWidth,
			(ScreenHeight DIV 24) * 23,
			SWP_NoZOrder);
END;

PROCEDURE TSysMetWin.Paint(DC: HDC; VAR PS: TPaintStruct);
BEGIN
	{ Draw our icon if needed }
	TWindow.Paint(DC, PS);
	if IsIconic(HWindow) then DrawMyIcon(DC);
END;

PROCEDURE TSysMetWin.DrawMyIcon(TheDC: hDC);
VAR	OldBrush, Brush: hBrush;
BEGIN
        { Save old brush and select first color }
	OldBrush := SelectObject(TheDC, CreateSolidBrush(UserMeter^.BarColorRef));

        { Calculate coordinates for USER meter }
	BarHeight := Round(UserMeter^.GetProgress / 100  * MaxBarHeight);
        wBegin := Offset + (MaxBarHeight - BarHeight);
	Rectangle(TheDC, Pix, wBegin, Pix + BarWidth, wBegin + BarHeight);

        { Delete previous brush and select next color }
	DeleteObject(SelectObject(TheDC, CreateSolidBrush(GDIMeter^.BarColorRef)));

	{ Calculate coordinates for GDI meter }
	BarHeight := Round(GDIMeter^.GetProgress / 100  * MaxBarHeight);
	wBegin := Offset + (MaxBarHeight - BarHeight);
	Rectangle(TheDC, (2 * Pix) + BarWidth, wBegin, (2 * Pix) + (2 * BarWidth),
		wBegin + BarHeight);

	{ Delete previous brush and select last color }
	DeleteObject(SelectObject(TheDC, CreateSolidBrush(MemMeter^.BarColorRef)));

	{ Calculate coordinates for Memory meter }
	BarHeight := Round(MemMeter^.GetProgress / 100  * MaxBarHeight);
	wBegin := Offset + (MaxBarHeight - BarHeight);
	Rectangle(TheDC, (3 * Pix) + (2 * BarWidth), wBegin,
		(3 * Pix) + (3 * BarWidth), wBegin + BarHeight);

	{ Delete last brush and select the old one we saved }
	DeleteObject(SelectObject(TheDC, OldBrush));
END;

PROCEDURE TSysMetWin.WMSysCommand(VAR Msg: TMessage);
BEGIN
	case Msg.wParam of
		CM_About: CMAbout(Msg);
		SC_Minimize:
			begin
                        { Minimize window }
			ShowWindow(HWindow, SW_Minimize);

			{ Make alternate window title } 
			MakeAltWindowTitle;

			{ And display it for visibility of values in TASKMAN }
			SetWindowText(HWindow, AltWindowTitle);
			end;
		else DefWndProc(Msg);
		end;
END;

PROCEDURE TSysMetWin.WMNCRButtonDown(VAR Msg: TMessage);
BEGIN
        { Make sure Object Inspector works in non-client area }
	WMRButtonDown(Msg);
END;

PROCEDURE TSysMetWin.WMRButtonDown(VAR Msg: TMessage);
VAR	Point: TPoint;
BEGIN
        { Pop up Object Inspector's local menu }
	Point := MakePoint(Msg.lParam);
	ClientToScreen(HWindow,Point);
        TrackPopupMenu(Menu,0,Point.X,Point.Y,0,HWindow,NIL);
END;

PROCEDURE TSysMetWin.CMAbout(VAR Msg: TMessage);
BEGIN
        { Pop up About dialog box }
	Application^.ExecDialog(New(PAboutDialog,Init(@Self,'About')));
END;

PROCEDURE TSysMetWin.CMPurpose(VAR Msg: TMessage);
BEGIN
	{ Pop up Purpose dialog box }
	Application^.ExecDialog(New(PCenterDialog,Init(@Self,'Purpose')));
END;

PROCEDURE TSysMetWin.CMCredits(VAR Msg: TMessage);
BEGIN
	{ Pop up Credits dialog box }
	Application^.ExecDialog(New(PCenterDialog,Init(@Self,'Credits')));
END;

PROCEDURE TSysMetWin.CMQuit(VAR Msg: TMessage);
BEGIN
        { Uh ... quit, I guess (I'm sure you could've figured that out) }
	CloseWindow;
END;

(* -------------------------------------------------------------------------- *)

{ TSysMetApp }

PROCEDURE TSysMetApp.InitMainWindow;
BEGIN
	MainWindow := New(PSysMetWin,Init(nil,AppName));
END;

(* -------------------------------------------------------------------------- *)

VAR     SysMetApp: TSysMetApp;

BEGIN
	SysMetApp.Init(AppName);
	SysMetApp.Run;
	SysMetApp.Done;
END.
